/*
 * dbcache.h, PFC d'en Guillem Cantallops Ramis a la UIB
 *
 * Copyright (C) 2000-2001, GCR.
 *
 * Distributed under the terms of the GNU GPL
 *
 */



/* Funci per mostrar alguns errors del MySQL*/

#define MYSQL_ERR(M,N)  { printf("%s[%d]: %s: %s\n", __FILE__, __LINE__, N, mysql_error(M)); exit(-1); }



/* Constants per la connexio a la BD MySQL, amb noms molt descriptius ;-D */
 
#define HOST		"localhost"
#define USER		"guillem"
#define PASSWD		"adfgvx"
#define DB			"monitor"
#define PORT		0
#define UNIX_SOCKET	"/var/lib/mysql/mysql.sock"
#define CLIENT_FLAG	0
 
#define MAX_QUERY_LEN	1024	/* Aix s una cosa meva, no ho limita el MySQL */



/* Temps (en segons) que passara dormit el thread gravador entre gravaci i gravaci */

#define GRAVADOR_SLEEP	15



/* Constants pel hashing */

#define HASH_BITS	8						/* Tamany del hash en bits (original: 10) */
#define HASH_SIZE	(1UL<<HASH_BITS)		/* La taula te 2^HASH_BITS entrades (original: 1024) */
#define HASH_MASK	(HASH_SIZE-1)			/* Mascara de HASH_BITS uns abaix (original: 0x3ff) */

#define MAX_LL_LEN	4						/* Maxim d'elements a un "bucket" (promig, si limitam el segent) */
#define MAX_HT_LEN	(HASH_SIZE*MAX_LL_LEN)	/* Maxim d'elements a la cach d'una taula (usat al CLOCK) */



/* HASH(KEY) simplement trunca la clau a un tamany adequat perque faci de hash,
   pero si fem servir varies claus ho podem arreglar molt facil i eficientment:
   noms li hem de passar la XOR de tots ells ;-) */

#define HASH(KEY)	(HASH_MASK&(KEY))



/* Aix s pel meu cas particular, per les taules que faig servir: */

#define HH(IP, MACH, MACL)						HASH(IP^MACH^MACL)
#define HO(IP, MACH, MACL, PORT, PROTOCOL)		HASH(IP^MACH^MACL^PORT^PROTOCOL)
#define HS(PORT, PROTOCOL)						HASH(PORT^PROTOCOL)



/* Aquesta s l'estructura per construir les taules de hash, igualment particularitzada.
   Usarem taules de HASH_SIZE elements d'aquests. Inicialitzar amb memset() a tot-zeros.*/

typedef struct node_host {
	u_long	ip;						/* Dades d'una entrada de la taula HOST */
	u_long	mach;
	u_long	macl;
	time_t	omega;
	time_t	alfa;
	u_char	ghost:1,				/* Dades noves -> encara s'han d'insertar a la BD */
			dirty:1,				/* Dades modificades -> s'han d'actualitzar a la BD */
			used:1,					/* Dades usades recentment -> per seleccionar victimes */
			reserved:5;				/* Aquests 5 bits sobren, de moment */
	struct node_host	*next;		/* Punter al segent element del bucket */
	struct node_host	*clnext;	/* Punter al segent element de la cach de la taula (usat al CLOCK) */
} NodeHost;

#define		NHKLEN				(3*sizeof(u_long))
#define		NHCMP(NH1, NH2)		(memcmp((const void *)NH1, (const void *)NH2, (size_t)NHKLEN))


typedef struct node_obert {
	u_long	ip;						/* Dades d'una entrada de la taula OBERT */
	u_long	mach;
	u_long	macl;
	u_short	port;
	u_char	protocol;
	time_t	omega;
	time_t	alfa;
	u_char	ghost:1,				/* Dades noves -> encara s'han d'insertar a la BD */
			dirty:1,				/* Dades modificades -> s'han d'actualitzar a la BD */
			used:1,					/* Dades usades recentment -> per seleccionar victimes */
			reserved:5;				/* Aquests 5 bits sobren, de moment */
	struct node_obert 	*next;		/* Punter al segent element del bucket */
	struct node_obert	*clnext;	/* Punter al segent element de la cach de la taula (usat al CLOCK) */
} NodeObert;

#define		NOKLEN				(3*sizeof(u_long)+sizeof(u_short)+sizeof(u_char))
#define		NOCMP(NO1, NO2)		(memcmp((const void *)NO1, (const void *)NO2, (size_t)NOKLEN))


typedef struct node_servei {
	u_short	port;					/* Dades d'una entrada de la taula SERVEI */
	u_char	protocol;
	struct node_servei	*next;		/* Punter al segent element del bucket */
} NodeServei;

#define		NSKLEN				(sizeof(u_short)+sizeof(u_char))
#define		NSCMP(NS1, NS2)		(memcmp((const void *)NS1, (const void *)NS2, (size_t)NSKLEN))

/* ALERTA: a node_servei ens hem deixat el camp "nom" de la taula "servei" i els flags perque no ens fan falta,
   la taula ser "read only", i a ms el camp aquest que no s'usa s de tipus "varchar" que s molt pesat :-P */



/* Funcions que proporcionam (veure dbcache.c per ms detalls :-) */

inline void print_cache_stats (void);
inline void init_dbcache (void);
inline void close_dbcache (void);
inline void insertar_host (u_int ip, u_int mach, u_int macl);
inline void actualitzar_omega_host (u_int ip, u_int mach, u_int macl);
inline void insertar_obert (u_int ip, u_int mach, u_int macl, u_short port, u_char protocol);
inline void actualitzar_omega_obert (u_int ip, u_int mach, u_int macl, u_short port, u_char protocol);
inline int hi_es_el_host (u_int ip, u_int mach, u_int macl);
inline int hi_es_el_servei (u_short port, u_char protocol);
inline int esta_obert_i_ho_sabem (u_int ip, u_int mach, u_int macl, u_short port, u_char protocol);
inline void gravador (void);


